@@ -26,7 +26,7 @@
 #include "update.h"
 
 #define MAX_NODES 256
-#define DEFAULT_CONFIG_FILE "/etc/cluster/cluster.conf"
+
 char *prog_name = "ccs_tool";
 
 #define die(fmt, args...) \
@@ -54,7 +54,7 @@ struct option_info
 
 static void config_usage(int rw)
 {
-	fprintf(stderr, " -c --configfile    Name of configuration file (/etc/cluster/cluster.conf)\n");
+	fprintf(stderr, " -c --configfile    Name of configuration file (" DEFAULT_CONFIG_DIR "/" DEFAULT_CONFIG_FILE ")\n");
 	if (rw)
 	{
 		fprintf(stderr, " -o --outputfile    Name of output file (defaults to same as --configfile)\n");
@@ -417,7 +417,7 @@ static xmlNode *find_node(xmlNode *clusternodes, char *nodename)
 /* Print name=value pairs for a (n XML) node.
  * "ignore" is a string to ignore if present as a property (probably already printed on the main line)
  */
-static void print_properties(xmlNode *node, char *prefix, char *ignore, char *ignore2)
+static int print_properties(xmlNode *node, char *prefix, char *ignore, char *ignore2)
 {
 	xmlAttr *attr;
 	int done_prefix = 0;
@@ -440,6 +440,7 @@ static void print_properties(xmlNode *node, char *prefix, char *ignore, char *ig
 	}
 	if (done_prefix)
 		printf("\n");
+	return done_prefix;
 }
 
 /* Add name=value pairs from the commandline as properties to a node */
@@ -477,6 +478,7 @@ static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
 {
 	xmlNode *clusternodes;
 	xmlNode *newnode;
+
 	xmlNode *newfence;
 	xmlNode *newfencemethod;
 	xmlNode *newfencedevice;
@@ -507,6 +509,15 @@ static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
 	xmlSetProp(newnode, BAD_CAST "nodeid", BAD_CAST ninfo->nodeid);
 	xmlAddChild(clusternodes, newnode);
 
+	if (ninfo->altname)
+	{
+		xmlNode *altnode;
+
+		altnode = xmlNewNode(NULL, BAD_CAST "altname");
+		xmlSetProp(altnode, BAD_CAST "name", BAD_CAST ninfo->altname);
+		xmlAddChild(newnode, altnode);
+	}
+
 	/* Add the fence attributes */
 	newfence = xmlNewNode(NULL, BAD_CAST "fence");
 	newfencemethod = xmlNewNode(NULL, BAD_CAST "method");
@@ -532,14 +543,14 @@ static xmlDoc *open_configfile(struct option_info *ninfo)
 	LIBXML_TEST_VERSION;
 
 	if (!ninfo->configfile)
-		ninfo->configfile = DEFAULT_CONFIG_FILE;
+		ninfo->configfile = DEFAULT_CONFIG_DIR "/" DEFAULT_CONFIG_FILE;
 	if (!ninfo->outputfile)
 		ninfo->outputfile = ninfo->configfile;
 
 	/* Load XML document */
 	doc = xmlParseFile(ninfo->configfile);
 	if (doc == NULL)
-		die("Error: unable to parse cluster.conf file\n");
+		die("Error: unable to parse requested configuration file\n");
 
 	return doc;
 
@@ -975,6 +986,13 @@ void list_nodes(int argc, char **argv)
 			       ftype?ftype:(xmlChar *)"");
 			if (verbose)
 			{
+				xmlNode *a = findnode(cur_node, "altname");
+				if (a)
+				{
+					printf(" altname %s=%s", "name", xmlGetProp(a, BAD_CAST "name"));
+					if (!print_properties(a, "","",""))
+						printf("\n");
+				}
 				print_properties(cur_node, "  Node properties: ", "votes", "nodeid");
 				print_properties(fencenode, "  Fence properties: ", "agent", "");
 			}
@@ -1018,7 +1036,7 @@ void create_skeleton(int argc, char **argv)
 		}
 	}
 	if (!ninfo.outputfile)
-		ninfo.outputfile = DEFAULT_CONFIG_FILE;
+		ninfo.outputfile = DEFAULT_CONFIG_DIR "/" DEFAULT_CONFIG_FILE;
 	ninfo.configfile = "-";
 
 	if (argc - optind < 1)
