@@ -170,8 +170,6 @@ int main(int argc, char *argv[]){
     }
   }
  
-  listen(sfds[1], 5);
-
   /* Set up the unix (local) socket for CCS lib comms */
   sfds[2] = setup_local_socket(SOMAXCONN);
 
@@ -273,7 +271,7 @@ static void print_usage(FILE *stream){
 	  " -t <ttl>      Multicast threshold (aka Time to Live) value.\n"
 	  " -P [bcf]:#    Specify various port numbers.\n"
 	  " -V            Print version information.\n"
-	  " -X            No cluster manager, just read local cluster.conf.\n"
+	  " -X            No cluster manager, just read local " DEFAULT_CONFIG_DIR "/" DEFAULT_CONFIG_FILE ".\n"
 	  );
   EXIT("print_usage");
 }
@@ -321,7 +319,7 @@ static char *parse_cli_args(int argc, char *argv[]){
 
   ENTER("parse_cli_args");
 
-  config_file_location = strdup(DEFAULT_CONFIG_LOCATION);
+  config_file_location = strdup(DEFAULT_CONFIG_DIR "/" DEFAULT_CONFIG_FILE);
   lockfile_location = strdup(DEFAULT_CCSD_LOCKFILE);
 
   if(!config_file_location || !lockfile_location){
@@ -478,7 +476,7 @@ static char *parse_cli_args(int argc, char *argv[]){
       ttl = atoi(optarg);
       break;
     case 'V':
-      printf("%s %s (built %s %s)\n", argv[0], CCS_RELEASE_NAME, __DATE__, __TIME__);
+      printf("%s %s (built %s %s)\n", argv[0], RELEASE_VERSION, __DATE__, __TIME__);
       printf("%s\n", REDHAT_COPYRIGHT);
       exit(EXIT_SUCCESS);
     case 'X':
@@ -532,7 +530,7 @@ static int check_cluster_conf(void){
   } else {
     /* no cluster.conf file.  This is fine, just need to get it from the network */
     if(no_manager_opt){
-      log_err("\nNo local cluster.conf found: %s\n", config_file_location);
+      log_err("\nNo local config file found: %s\n", config_file_location);
       return -1;
     }
   }
@@ -784,7 +782,7 @@ static void daemonize(void){
  */
 static void print_start_msg(char *msg){
   /* We want the start message to print every time */
-  log_msg("Starting ccsd %s:\n", CCS_RELEASE_NAME);
+  log_msg("Starting ccsd %s:\n", RELEASE_VERSION);
   log_msg(" Built: "__DATE__" "__TIME__"\n");
   log_msg(" %s\n", REDHAT_COPYRIGHT);
   if(msg){
