@@ -28,6 +28,7 @@
 #include <netdb.h>
 
 /* openais headers */
+#include <openais/service/objdb.h>
 #include <openais/service/swab.h>
 #include <openais/totem/totemip.h>
 #include <openais/totem/totempg.h>
@@ -36,9 +37,10 @@
 #include <openais/service/config.h>
 #include <openais/lcr/lcr_comp.h>
 #include <openais/service/swab.h>
-#include <openais/service/print.h>
 
+#include "list.h"
 #include "cnxman-socket.h"
+#include "cnxman-private.h"
 #include "commands.h"
 #include "logging.h"
 
@@ -61,7 +63,6 @@ uint64_t incarnation;
 int num_ais_nodes;
 
 static int config_run;
-static char errorstring[512];
 static int startup_pipe;
 static unsigned int debug_mask;
 static int first_trans = 1;
@@ -71,6 +72,8 @@ static struct totempg_group cman_group[1] = {
         { .group          = "CMAN", .group_len      = 4},
 };
 
+LOGSYS_DECLARE_SUBSYS (CMAN_NAME, LOG_INFO);
+
 /* This structure is tacked onto the start of a cluster message packet for our
  * own nefarious purposes. */
 struct cl_protheader {
@@ -135,7 +138,7 @@ static inline void objdb_get_int(struct objdb_iface_ver0 *objdb, unsigned int ob
  * Exports the interface for the service
  */
 static struct openais_service_handler cman_service_handler = {
-	.name		    		= (char *)"openais CMAN membership service 2.01",
+	.name		    		= (unsigned char *)"openais CMAN membership service 2.01",
 	.id			        = CMAN_SERVICE,
 	.flow_control			= OPENAIS_FLOW_CONTROL_NOT_REQUIRED,
 	.lib_exit_fn		       	= cman_exit_fn,
@@ -204,20 +207,22 @@ static int cman_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
 
 	/* Initialise early logging */
 	if (getenv("CMAN_DEBUGLOG"))
-	{
 		debug_mask = atoi(getenv("CMAN_DEBUGLOG"));
-	}
 
 	if (getenv("CMAN_PIPE"))
 		startup_pipe = atoi(getenv("CMAN_PIPE"));
 
-	init_debug(debug_mask);
+	set_debuglog(debug_mask);
 
 	/* We need to set this up to internal defaults too early */
 	openlog("openais", LOG_CONS|LOG_PID, LOG_LOCAL4);
 
 	global_objdb = objdb;
 
+	/* Enable stderr logging if requested by cman_tool */
+	if (debug_mask)
+		logsys_config_subsys_set(CMAN_NAME, LOGSYS_TAG_LOG, LOG_DEBUG);
+
 	/* Read low-level totem/aisexec etc config from CCS */
 	init_config(objdb);
 
@@ -226,7 +231,7 @@ static int cman_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
 	if (error)
 	{
 		write_cman_pipe("Error reading config from CCS");
-		sprintf(errorstring, "Error reading config from CCS");
+		*error_string = "Error reading config from CCS";
 		return -1;
 	}
 
@@ -235,14 +240,17 @@ static int cman_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
 
 	config_run = 1;
 
+	*error_string = "Successfully read configuration from CCS";
+
 	return 0;
 }
 
 static int cman_exec_init_fn(struct objdb_iface_ver0 *objdb)
 {
 	unsigned int object_handle;
+	char pipe_msg[256];
 
-	/* We can only work if our config inerface was run first */
+	/* We can only work if our config interface was run first */
 	if (!config_run)
 		return 0;
 
@@ -258,14 +266,16 @@ static int cman_exec_init_fn(struct objdb_iface_ver0 *objdb)
 		if (!getenv("CMAN_DEBUGLOG"))
 		{
 			objdb_get_int(objdb, object_handle, "debug_mask", &debug_mask);
-			init_debug(debug_mask);
+			set_debuglog(debug_mask);
 		}
 	}
 
 	/* Open local sockets and initialise I/O queues */
 	cman_init();
 
-	/* Let cman_tool know we are running */
+	/* Let cman_tool know we are running and our PID */
+	sprintf(pipe_msg,"SUCCESS: %d", getpid());
+	write_cman_pipe(pipe_msg);
 	close(startup_pipe);
 	startup_pipe = 0;
 
@@ -295,11 +305,17 @@ int ais_add_ifaddr(char *mcast, char *ifaddr, int portnum)
 
 	P_AIS("Adding local address %s\n", ifaddr);
 
-	/* This will already exist as early config creates it */
 	global_objdb->object_find_reset(OBJECT_PARENT_HANDLE);
 	if (global_objdb->object_find(OBJECT_PARENT_HANDLE,
-				      "totem", strlen("totem"),&totem_object_handle) == 0) {
+				       "totem", strlen("totem"), &totem_object_handle)) {
 
+		global_objdb->object_create(OBJECT_PARENT_HANDLE, &totem_object_handle,
+					    "totem", strlen("totem"));
+	}
+
+	global_objdb->object_find_reset(OBJECT_PARENT_HANDLE);
+	if (global_objdb->object_find(OBJECT_PARENT_HANDLE,
+				      "totem", strlen("totem"), &totem_object_handle) == 0) {
 		if (global_objdb->object_create(totem_object_handle, &interface_object_handle,
 						"interface", strlen("interface")) == 0) {
 
@@ -507,13 +523,15 @@ static int comms_init_ais(struct objdb_iface_ver0 *objdb)
 
 
 		/* Set RRP mode appropriately */
-		if (num_interfaces > 1) {
-			global_objdb->object_key_create(object_handle, "rrp_mode", strlen("rrp_mode"),
-							"active", strlen("active")+1);
-		}
-		else {
-			global_objdb->object_key_create(object_handle, "rrp_mode", strlen("rrp_mode"),
-							"none", strlen("none")+1);
+		if (objdb_get_string(objdb, object_handle, "rrp_mode", &value)) {
+			if (num_interfaces > 1) {
+				global_objdb->object_key_create(object_handle, "rrp_mode", strlen("rrp_mode"),
+								"active", strlen("active")+1);
+			}
+			else {
+				global_objdb->object_key_create(object_handle, "rrp_mode", strlen("rrp_mode"),
+								"none", strlen("none")+1);
+			}
 		}
 
 		if (objdb_get_string(objdb, object_handle, "secauth", &value)) {
@@ -546,6 +564,13 @@ static int comms_init_ais(struct objdb_iface_ver0 *objdb)
 	}
 
 	/* Make sure mainconfig doesn't stomp on our logging options */
+	if (global_objdb->object_find(OBJECT_PARENT_HANDLE,
+				      "logging", strlen("logging"), &object_handle)) {
+
+		global_objdb->object_create(OBJECT_PARENT_HANDLE, &object_handle,
+					    "logging", strlen("logging"));
+	}
+
 	objdb->object_find_reset(OBJECT_PARENT_HANDLE);
 	if (objdb->object_find(OBJECT_PARENT_HANDLE,
 			       "logging", strlen("logging"),
@@ -561,20 +586,14 @@ static int comms_init_ais(struct objdb_iface_ver0 *objdb)
 		}
 
 		objdb->object_create(object_handle, &logger_object_handle,
-				      "logger", strlen("logger"));
-		objdb->object_key_create(logger_object_handle, "ident", strlen("ident"),
-					 "CMAN", strlen("CMAN")+1);
+				      "logger_subsys", strlen("logger_subsys"));
+		objdb->object_key_create(logger_object_handle, "subsys", strlen("subsys"),
+					 CMAN_NAME, strlen(CMAN_NAME)+1);
 
-		if (debug_mask)
-		{
-			objdb->object_key_create(object_handle, "to_stderr", strlen("to_stderr"),
-						 "yes", strlen("yes")+1);
+		if (debug_mask) {
 			objdb->object_key_create(logger_object_handle, "debug", strlen("debug"),
 						 "on", strlen("on")+1);
-		}
-		else
-		{
-			objdb->object_key_create(object_handle, "to_syslog", strlen("to_syslog"),
+			objdb->object_key_create(object_handle, "to_stderr", strlen("to_stderr"),
 						 "yes", strlen("yes")+1);
 		}
 	}
