@@ -21,6 +21,7 @@
 #include <time.h>
 #include <signal.h>
 #include <syslog.h>
+#include <sys/time.h>
 #include <asm/types.h>
 #include <sys/socket.h>
 #include <sys/poll.h>
@@ -54,22 +55,43 @@ static int stress_delay = 0;
 static int stress_lock_only = 0;
 static int openclose_ls = 0;
 static uint64_t our_xid;
+static char cmd[32];
+static int opt_cmd = 0;
 
 static unsigned int sts_eunlock, sts_ecancel, sts_etimedout, sts_edeadlk, sts_eagain, sts_other, sts_zero;
 static unsigned int bast_unlock, bast_skip;
 
+
 #define log_print(fmt, args...) \
 do { \
 	if (!quiet) \
 		printf(fmt , ##args); \
 } while (0)
 
-#define log_verbose(fmt, args...) \
+#define log_op(fmt, args...) \
+do { \
+	if (!quiet) \
+		printf(fmt , ##args); \
+} while (0)
+
+#define log_ast(fmt, args...) \
 do { \
 	if (verbose) \
 		printf(fmt , ##args); \
 } while (0)
 
+#define log_bast(fmt, args...) \
+do { \
+	if (verbose > 1) \
+		printf(fmt , ##args); \
+} while (0)
+
+#define log_verbose(fmt, args...) \
+do { \
+	if (verbose > 2) \
+		printf(fmt , ##args); \
+} while (0)
+
 struct client {
 	int fd;
 	char type[32];
@@ -102,16 +124,16 @@ struct lk {
 
 struct lk *locks;
 
-void unlock(int i);
-void unlockf(int i);
+static void unlock(int i);
+static void unlockf(int i);
 
 
-int rand_int(int a, int b)
+static int rand_int(int a, int b)
 {
 	return a + (int) (((float)(b - a + 1)) * random() / (RAND_MAX+1.0)); 
 }
 
-char *status_str(int status)
+static char *status_str(int status)
 {
 	static char sts_str[8];
 
@@ -136,7 +158,7 @@ char *status_str(int status)
 	}
 }
 
-char *op_str(int op)
+static char *op_str(int op)
 {
 	switch (op) {
 	case Op_lock:
@@ -152,7 +174,7 @@ char *op_str(int op)
 	}
 }
 
-struct lk *get_lock(int i)
+static struct lk *get_lock(int i)
 {
 	if (i < 0)
 		return NULL;
@@ -161,7 +183,7 @@ struct lk *get_lock(int i)
 	return &locks[i];
 }
 
-int all_unlocks_done(void)
+static int all_unlocks_done(void)
 {
 	struct lk *lk;
 	int i;
@@ -175,7 +197,7 @@ int all_unlocks_done(void)
 	return 1;
 }
 
-void dump(void)
+static void dump(void)
 {
 	struct timeval now;
 	struct lk *lk;
@@ -193,18 +215,18 @@ void dump(void)
 			lk->wait_ast,
 			op_str(lk->lastop),
 			status_str(lk->last_status),
-			lk->wait_ast ? now.tv_sec - lk->begin.tv_sec : 0);
+			lk->wait_ast ? (unsigned int)(now.tv_sec - lk->begin.tv_sec) : 0);
 	}
 }
 
-void bastfn(void *arg)
+static void bastfn(void *arg)
 {
 	struct lk *lk = arg;
 	lk->bast = 1;
 	bast_cb = 1;
 }
 
-void do_bast(struct lk *lk)
+static void do_bast(struct lk *lk)
 {
 	int skip = 0;
 
@@ -217,16 +239,16 @@ void do_bast(struct lk *lk)
 
 	if (skip) {
 		bast_skip++;
-		printf("    bast: skip    %3d\t%x\n", lk->id, lk->lksb.sb_lkid);
+		log_bast("    bast: skip    %3d\t%x\n", lk->id, lk->lksb.sb_lkid);
 	} else {
 		bast_unlock++;
-		printf("    bast: unlockf %3d\t%x\n", lk->id, lk->lksb.sb_lkid);
+		log_bast("    bast: unlockf %3d\t%x\n", lk->id, lk->lksb.sb_lkid);
 		unlockf(lk->id);
 	}
 	lk->bast = 0;
 }
 
-void do_bast_unlocks(void)
+static void do_bast_unlocks(void)
 {
 	struct lk *lk;
 	int i;
@@ -239,14 +261,14 @@ void do_bast_unlocks(void)
 	bast_cb = 0;
 }
 
-void process_libdlm(void)
+static void process_libdlm(void)
 {
 	dlm_dispatch(libdlm_fd);
 	if (bast_cb && !ignore_bast)
 		do_bast_unlocks();
 }
 
-void astfn(void *arg)
+static void astfn(void *arg)
 {
 	struct lk *lk = arg;
 	int i = lk->id;
@@ -258,8 +280,8 @@ void astfn(void *arg)
 		       op_str(lk->lastop), status_str(lk->last_status));
 	}
 
-	log_print("     ast: %s %3d\t%x\n",
-		  status_str(lk->lksb.sb_status), i, lk->lksb.sb_lkid);
+	log_ast("     ast: %s %3d\t%x\n",
+		status_str(lk->lksb.sb_status), i, lk->lksb.sb_lkid);
 
 	lk->last_status = lk->lksb.sb_status;
 
@@ -341,7 +363,7 @@ void astfn(void *arg)
    and issues the second lock before the reply for the overlapping
    cancel (which did nothing) has been received in the dlm. */
 
-void lock(int i, int mode)
+static void lock(int i, int mode)
 {
 	char name[DLM_RESNAME_MAXLEN];
 	struct lk *lk;
@@ -397,7 +419,7 @@ void lock(int i, int mode)
 	lk->lastop = Op_lock;
 }
 
-void lock_sync(int i, int mode)
+static void lock_sync(int i, int mode)
 {
 	char name[DLM_RESNAME_MAXLEN];
 	int flags = 0;
@@ -441,7 +463,7 @@ void lock_sync(int i, int mode)
 	}
 }
 
-void lock_all(int mode)
+static void lock_all(int mode)
 {
 	int i;
 
@@ -449,7 +471,7 @@ void lock_all(int mode)
 		lock(i, mode);
 }
 
-char *uflags(uint32_t flags)
+static char *uflags(uint32_t flags)
 {
 	if (flags == LKF_FORCEUNLOCK)
 		return "FORCEUNLOCK";
@@ -462,7 +484,7 @@ char *uflags(uint32_t flags)
    try to do an unlockf during an outstanding op that will free
    the lock itself */
 
-void _unlock(int i, uint32_t flags)
+static void _unlock(int i, uint32_t flags)
 {
 	struct lk *lk;
 	uint32_t lkid;
@@ -494,7 +516,7 @@ void _unlock(int i, uint32_t flags)
 	}
 }
 
-void unlock(int i)
+static void unlock(int i)
 {
 	struct lk *lk = get_lock(i);
 
@@ -508,7 +530,7 @@ void unlock(int i)
 		if (lk->acquired.tv_sec + lk->minhold > now.tv_sec) {
 			printf("        : unlock  %3d\t%x: gr %d rq %d held %u of %u s\n",
 				i, lk->lksb.sb_lkid, lk->grmode, lk->rqmode,
-				now.tv_sec - lk->acquired.tv_sec, lk->minhold);
+				(unsigned int)(now.tv_sec - lk->acquired.tv_sec), lk->minhold);
 			return;
 		}
 	}
@@ -518,7 +540,7 @@ void unlock(int i)
 	lk->lastop = Op_unlock;
 }
 
-void unlockf(int i)
+static void unlockf(int i)
 {
 	struct lk *lk = get_lock(i);
 
@@ -532,7 +554,7 @@ void unlockf(int i)
 		if (lk->acquired.tv_sec + lk->minhold > now.tv_sec) {
 			printf("        : unlockf %3d\t%x: gr %d rq %d held %u of %u s\n",
 				i, lk->lksb.sb_lkid, lk->grmode, lk->rqmode,
-				now.tv_sec - lk->acquired.tv_sec, lk->minhold);
+				(unsigned int)(now.tv_sec - lk->acquired.tv_sec), lk->minhold);
 			return;
 		}
 	}
@@ -542,14 +564,14 @@ void unlockf(int i)
 	lk->lastop = Op_unlockf;
 }
 
-void cancel(int i)
+static void cancel(int i)
 {
 	struct lk *lk = get_lock(i);
 	_unlock(i, LKF_CANCEL);
 	lk->lastop = Op_cancel;
 }
 
-void canceld(int i, uint32_t lkid)
+static void canceld(int i, uint32_t lkid)
 {
 	int rv;
 
@@ -558,7 +580,7 @@ void canceld(int i, uint32_t lkid)
 	printf("canceld %x: %d %d\n", lkid, rv, errno);
 }
 
-void unlock_sync(int i)
+static void unlock_sync(int i)
 {
 	uint32_t lkid;
 	int rv;
@@ -586,7 +608,7 @@ void unlock_sync(int i)
 	lk->rqmode = -1;
 }
 
-void unlock_all(void)
+static void unlock_all(void)
 {
 	struct lk *lk;
 	int i;
@@ -597,7 +619,7 @@ void unlock_all(void)
 	}
 }
 
-void purge(int nodeid, int pid)
+static void purge(int nodeid, int pid)
 {
 	struct lk *lk;
 	int i, rv;
@@ -616,7 +638,7 @@ void purge(int nodeid, int pid)
 	}
 }
 
-void purgetest(int nodeid, int pid)
+static void purgetest(int nodeid, int pid)
 {
 	struct lk *lk;
 	int i, mid = maxn / 2;
@@ -645,7 +667,7 @@ void purgetest(int nodeid, int pid)
 	purge(nodeid, pid);
 }
 
-void tstress_unlocks(void)
+static void tstress_unlocks(void)
 {
 	struct lk *lk;
 	struct timeval now;
@@ -667,7 +689,7 @@ void tstress_unlocks(void)
 		if (now.tv_sec >= lk->acquired.tv_sec + minhold) {
 			printf("        : unlock  %3d\t%x: gr %d rq %d held %u of %u s\n",
 				i, lk->lksb.sb_lkid, lk->grmode, lk->rqmode,
-				now.tv_sec - lk->acquired.tv_sec, minhold);
+				(unsigned int)(now.tv_sec - lk->acquired.tv_sec), minhold);
 
 			_unlock(i, 0);
 			lk->rqmode = -1;
@@ -677,10 +699,10 @@ void tstress_unlocks(void)
 	}
 }
 
-void tstress(int num)
+static void tstress(int num)
 {
-	int i, o, op, max_op, skip;
 	unsigned int n, skips, lock_ops, unlock_ops, unlockf_ops, cancel_ops;
+	int i;
 	struct lk *lk;
 
 	n = skips = lock_ops = unlock_ops = unlockf_ops = cancel_ops = 0;
@@ -732,7 +754,55 @@ void tstress(int num)
 	printf("ast status: zero %d other %d\n", sts_zero, sts_other);
 }
 
-void stress(int num)
+static void dstress(int num)
+{
+	unsigned int n, skips, lock_ops, unlock_ops, unlockf_ops, cancel_ops;
+	int i;
+	struct lk *lk;
+
+	n = skips = lock_ops = unlock_ops = unlockf_ops = cancel_ops = 0;
+	sts_eunlock = sts_ecancel = sts_etimedout = sts_edeadlk = sts_eagain = sts_other = sts_zero = 0;
+	bast_unlock = bast_skip = 0;
+
+	noqueue = 0;
+	ignore_bast = 1;
+	quiet = 0;
+
+	while (!stress_stop && n < num) {
+
+		sleep(1);
+
+		process_libdlm();
+
+		if (n && !(n % 60))
+			unlock_all();
+		n++;
+
+		i = rand_int(0, maxn-1);
+		lk = get_lock(i);
+		if (!lk)
+			continue;
+
+		if (lk->wait_ast || lk->grmode > -1) {
+			printf("%8x: lock    %3d\t%x: skip gr %d wait_ast %d\n",
+				n, i, lk->lksb.sb_lkid, lk->grmode, lk->wait_ast);
+			continue;
+		}
+
+		lock(i, rand_int(0, 5));
+		lock_ops++;
+		printf("%8x: lock    %3d\t%x\n", n, i, lk->lksb.sb_lkid);
+	}
+
+	printf("ops: skip %d lock %d unlock %d unlockf %d cancel %d\n",
+		skips, lock_ops, unlock_ops, unlockf_ops, cancel_ops);
+	printf("bast: unlock %u skip %u\n", bast_unlock, bast_skip);
+	printf("ast status: eunlock %d ecancel %d etimedout %d edeadlk %d eagain %d\n",
+		sts_eunlock, sts_ecancel, sts_etimedout, sts_edeadlk, sts_eagain);
+	printf("ast status: zero %d other %d\n", sts_zero, sts_other);
+}
+
+static void stress(int num)
 {
 	int i, o, op, max_op, skip;
 	unsigned int n, skips, lock_ops, unlock_ops, unlockf_ops, cancel_ops;
@@ -776,6 +846,8 @@ void stress(int num)
 		case 5:
 			op = Op_cancel;
 			break;
+		default:
+			op = 0;
 		}
 
 		skip = 0;
@@ -792,7 +864,7 @@ void stress(int num)
 
 			lock(i, rand_int(0, 5));
 			lock_ops++;
-			printf("%8x: lock    %3d\t%x\n", n, i, lk->lksb.sb_lkid);
+			log_op("%8x: lock    %3d\t%x\n", n, i, lk->lksb.sb_lkid);
 			break;
 
 		case Op_unlock:
@@ -811,7 +883,7 @@ void stress(int num)
 
 			unlock(i);
 			unlock_ops++;
-			printf("%8x: unlock  %3d\t%x\n", n, i, lk->lksb.sb_lkid);
+			log_op("%8x: unlock  %3d\t%x\n", n, i, lk->lksb.sb_lkid);
 			break;
 
 		case Op_unlockf:
@@ -826,7 +898,7 @@ void stress(int num)
 
 			unlockf(i);
 			unlockf_ops++;
-			printf("%8x: unlockf %3d\t%x\n", n, i, lk->lksb.sb_lkid);
+			log_op("%8x: unlockf %3d\t%x\n", n, i, lk->lksb.sb_lkid);
 			break;
 
 		case Op_cancel:
@@ -841,7 +913,7 @@ void stress(int num)
 
 			cancel(i);
 			cancel_ops++;
-			printf("%8x: cancel  %3d\t%x\n", n, i, lk->lksb.sb_lkid);
+			log_op("%8x: cancel  %3d\t%x\n", n, i, lk->lksb.sb_lkid);
 			break;
 		}
 
@@ -892,11 +964,11 @@ static void client_init(void)
 		client[i].fd = -1;
 }
 
-void print_commands(void)
+static void print_commands(void)
 {
 	printf("Usage:\n");
 	printf("max locks (maxn) is %d (x of 0 to %d)\n", maxn, maxn-1);
-	printf("max resources (maxr) is %d, lock x used on resource (x % maxr)\n", maxr);
+	printf("max resources (maxr) is %d, lock x used on resource (x %% maxr)\n", maxr);
 	printf("EXIT		 - exit program after unlocking any held locks\n");
 	printf("kill		 - exit program without unlocking any locks\n");
 	printf("lock x mode	 - request/convert lock x\n");
@@ -913,6 +985,7 @@ void print_commands(void)
 	printf("purge nodeid pid - purge orphan locks of process\n");
 	printf("stress n	 - loop doing random lock/unlock/unlockf/cancel on all locks, n times\n");
 	printf("tstress n	 - stress timeouts\n");
+	printf("dstress n	 - stress deadlock\n");
 	printf("timeout n	 - enable lock timeouts, set timeout to n seconds\n");
 	printf("dump		 - show info for all locks\n");
 	printf("minhold		 - set minimum number of seconds locks will be held\n");
@@ -921,6 +994,7 @@ void print_commands(void)
 	printf("persistent	 - toggle PERSISTENT flag for all requests\n");
 	printf("quiet		 - toggle quiet flag\n");
 	printf("verbose		 - toggle verbose flag\n");
+	printf("settings	 - show settings\n");
 
 	printf("\ncombined operations\n");
 	printf("hold-kill                     - hold; kill\n");
@@ -934,7 +1008,7 @@ void print_commands(void)
 	printf("purgetest nodeid pid\n");
 }
 
-void print_settings(void)
+static void print_settings(void)
 {
 	printf("timewarn %d\n", timewarn);
 	printf("timeout %llu\n", (unsigned long long) timeout);
@@ -950,17 +1024,18 @@ void print_settings(void)
 	printf("stress_stop %d\n", stress_stop);
 	printf("stress_delay %d\n", stress_delay);
 	printf("stress_lock_only %d\n", stress_lock_only);
+	printf("our_xid %llx\n", (unsigned long long)our_xid);
 }
 
-void process_command(int *quit)
+static void process_command(int *quit)
 {
 	char inbuf[132];
-	char cmd[32];
-	int x = 0, y = 0, z = 0;
+	int x = 0, y = 0;
 
-	fgets(inbuf, sizeof(inbuf), stdin);
-
-	sscanf(inbuf, "%s %d %d", cmd, &x, &y, &z);
+	if (!opt_cmd) {
+		fgets(inbuf, sizeof(inbuf), stdin);
+		sscanf(inbuf, "%s %d %d", cmd, &x, &y);
+	}
 
 	if (!strncmp(cmd, "EXIT", 4)) {
 		*quit = 1;
@@ -1077,6 +1152,8 @@ void process_command(int *quit)
 	}
 
 	if (!strncmp(cmd, "stress", 6) && strlen(cmd) == 6) {
+		if (iterations && !x)
+			x = iterations;
 		stress(x);
 		return;
 	}
@@ -1086,6 +1163,11 @@ void process_command(int *quit)
 		return;
 	}
 
+	if (!strncmp(cmd, "dstress", 7) && strlen(cmd) == 7) {
+		dstress(x);
+		return;
+	}
+
 	if (!strncmp(cmd, "stress_delay", 12) && strlen(cmd) == 12) {
 		stress_delay = x;
 		return;
@@ -1097,12 +1179,25 @@ void process_command(int *quit)
 		return;
 	}
 
+	if (!strncmp(cmd, "stress_stop", 11) && strlen(cmd) == 11) {
+		stress_stop = !stress_stop;
+		printf("stress_stop is %d\n", stress_stop);
+		return;
+	}
+
 	if (!strncmp(cmd, "ignore_bast", 11) && strlen(cmd) == 11) {
 		ignore_bast = !ignore_bast;
 		printf("ignore_bast is %s\n", ignore_bast ? "on" : "off");
 		return;
 	}
 
+	if (!strncmp(cmd, "our_xid", 7) && strlen(cmd) == 7) {
+		our_xid = x;
+		printf("our_xid is %llx\n", (unsigned long long)our_xid);
+		return;
+	}
+
+
 	if (!strncmp(cmd, "purge", 5) && strlen(cmd) == 5) {
 		purge(x, y);
 		return;
@@ -1161,7 +1256,7 @@ void process_command(int *quit)
 	printf("unknown command %s\n", cmd);
 }
 
-void print_usage(void)
+static void print_usage(void)
 {
 	printf("Options:\n");
 	printf("\n");
@@ -1177,7 +1272,7 @@ static void decode_arguments(int argc, char **argv)
 	int optchar;
 
 	while (cont) {
-		optchar = getopt(argc, argv, "n:r:i:thVo");
+		optchar = getopt(argc, argv, "n:r:c:i:thVoq:v:");
 
 		switch (optchar) {
 
@@ -1189,6 +1284,11 @@ static void decode_arguments(int argc, char **argv)
 			maxr = atoi(optarg);
 			break;
 
+		case 'c':
+			strcpy(cmd, optarg);
+			opt_cmd = 1;
+			break;
+
 		case 'i':
 			iterations = atoi(optarg);
 			break;
@@ -1201,6 +1301,14 @@ static void decode_arguments(int argc, char **argv)
 			openclose_ls = 1;
 			break;
 
+		case 'q':
+			quiet = atoi(optarg);
+			break;
+
+		case 'v':
+			verbose = atoi(optarg);
+			break;
+
 		case 'h':
 			print_usage();
 			exit(EXIT_SUCCESS);
@@ -1241,6 +1349,8 @@ int main(int argc, char *argv[])
 	struct lk *lk;
 	int i, rv, maxi = 0, quit = 0;
 
+	srandom(time(NULL));
+
 	decode_arguments(argc, argv);
 
 	if (maxn < maxr) {
@@ -1263,7 +1373,7 @@ int main(int argc, char *argv[])
 	locks = malloc(maxn * sizeof(struct lk));
 	if (!locks) {
 		printf("no mem for %d locks\n", maxn);
-		return;
+		return 0;
 	}
 	memset(locks, 0, sizeof(*locks));
 
@@ -1296,7 +1406,8 @@ int main(int argc, char *argv[])
 	} else {
 		printf("dlm_new_lockspace...\n");
 
-		dh = dlm_new_lockspace("test", 0600, timewarn ? DLM_LSFL_TIMEWARN : 0);
+		dh = dlm_new_lockspace("test", 0600,
+				       timewarn ? DLM_LSFL_TIMEWARN : 0);
 		if (!dh) {
 			printf("dlm_new_lockspace error %lu %d\n",
 				(unsigned long)dh, errno);
@@ -1313,10 +1424,13 @@ int main(int argc, char *argv[])
 	libdlm_fd = rv;
 
 	client_add(libdlm_fd, &maxi);
-	client_add(STDIN_FILENO, &maxi);
 
-	if (strstr(argv[0], "dlmstress"))
-		stress(iterations);
+	if (cmd) {
+		process_command(&quit);
+		goto out;
+	}
+
+	client_add(STDIN_FILENO, &maxi);
 
 	printf("Type EXIT to finish, help for usage\n");
 
@@ -1346,6 +1460,7 @@ int main(int argc, char *argv[])
 			break;
 	}
 
+ out:
 	if (openclose_ls) {
 		printf("dlm_close_lockspace\n");
 
