@@ -1444,8 +1444,7 @@ blkalloc_internal(struct gfs_rgrpd *rgd,
 	   allocatable block anywhere else, we want to be able wrap around and
 	   search in the first part of our first-searched bit block.  */
 	for (x = 0; x <= length; x++) {
-		blk = gfs_bitfit(rgd,
-				 rgd->rd_bh[buf]->b_data + bits->bi_offset,
+		blk = gfs_bitfit(rgd->rd_bh[buf]->b_data + bits->bi_offset,
 				 bits->bi_len, goal, old_state);
 		if (blk != BFITNOENT)
 			break;
@@ -1456,8 +1455,13 @@ blkalloc_internal(struct gfs_rgrpd *rgd,
 		goal = 0;
 	}
 
-	if (gfs_assert_withdraw(rgd->rd_sbd, x <= length))
-		blk = 0;
+	if (unlikely(x > length)) {
+		printk("GFS error: possible RG corruption\n");
+		printk("    please run gfs_fsck after withdraw\n");
+		dump_stack();
+		if (gfs_assert_withdraw(rgd->rd_sbd, x <= length))
+			blk = 0;
+	}
 
 	/* Attach bitmap buffer to trans, modify bits to do block alloc */
 	gfs_trans_add_bh(rgd->rd_gl, rgd->rd_bh[buf]);
@@ -1501,7 +1505,7 @@ blkfree_internal(struct gfs_sbd *sdp, uint64_t bstart, uint32_t blen,
 	rgd = gfs_blk2rgrpd(sdp, bstart);
 	if (!rgd) {
 		if (gfs_consist(sdp))
-			printk("GFS: fsid=%s: block = %"PRIu64"\n",
+			printk("GFS: fsid=%s: block = %llu\n",
 			       sdp->sd_fsname, bstart);
 		return NULL;
 	}
@@ -1658,6 +1662,9 @@ gfs_blkalloc(struct gfs_inode *ip, uint64_t *block)
 	al->al_alloced_data++;
 
 	gfs_trans_add_quota(sdp, +1, ip->i_di.di_uid, ip->i_di.di_gid);
+
+	/* total=0, free=-1, dinodes=0 */
+	gfs_statfs_modify(sdp, 0, -1, 0);
 }
 
 /**
@@ -1712,6 +1719,9 @@ gfs_metaalloc(struct gfs_inode *ip, uint64_t *block)
 
 	gfs_trans_add_quota(sdp, +1, ip->i_di.di_uid, ip->i_di.di_gid);
 
+	/* total=0, free=-1, dinode=0 */
+	gfs_statfs_modify(sdp, 0, -1, 0);
+
 	return 0;
 }
 
@@ -1727,6 +1737,7 @@ gfs_metaalloc(struct gfs_inode *ip, uint64_t *block)
 int
 gfs_dialloc(struct gfs_inode *dip, uint64_t *block)
 {
+	struct gfs_sbd *sdp = dip->i_sbd;
 	struct gfs_alloc *al = dip->i_alloc;
 	struct gfs_rgrpd *rgd = al->al_rgd;
 	uint32_t goal, blk;
@@ -1766,6 +1777,9 @@ gfs_dialloc(struct gfs_inode *dip, uint64_t *block)
 	al->al_alloced_di++;
 	al->al_alloced_meta++;
 
+	/* total=0, free=-1, dinodes=1 */
+	gfs_statfs_modify(sdp, 0, -1, +1);
+
 	return error;
 }
 
@@ -1798,6 +1812,9 @@ gfs_blkfree(struct gfs_inode *ip, uint64_t bstart, uint32_t blen)
 	gfs_trans_add_quota(sdp, -(int64_t)blen,
 			    ip->i_di.di_uid,
 			    ip->i_di.di_gid);
+
+	/* total=0, free=+blen, dinodes=0 */
+	gfs_statfs_modify(sdp, 0, blen, 0);
 }
 
 /**
@@ -1832,6 +1849,9 @@ gfs_metafree(struct gfs_inode *ip, uint64_t bstart, uint32_t blen)
 	gfs_trans_add_bh(rgd->rd_gl, rgd->rd_bh[0]);
 	gfs_rgrp_out(&rgd->rd_rg, rgd->rd_bh[0]->b_data);
 
+	/* total=0, free=blen, dinode=0 */
+	gfs_statfs_modify(sdp, 0, blen, 0);
+
 	gfs_trans_add_quota(sdp, -(int64_t)blen,
 			    ip->i_di.di_uid,
 			    ip->i_di.di_gid);
@@ -1866,6 +1886,9 @@ gfs_difree_uninit(struct gfs_rgrpd *rgd, uint64_t addr)
 
 	gfs_trans_add_bh(rgd->rd_gl, rgd->rd_bh[0]);
 	gfs_rgrp_out(&rgd->rd_rg, rgd->rd_bh[0]->b_data);
+
+	/* total=0, free=1, dinodes=-1 */
+	gfs_statfs_modify(rgd->rd_sbd, 0, +1, -1);
 }
 
 /**
@@ -1914,7 +1937,7 @@ gfs_rlist_add(struct gfs_sbd *sdp, struct gfs_rgrp_list *rlist, uint64_t block)
 	rgd = gfs_blk2rgrpd(sdp, block);
 	if (!rgd) {
 		if (gfs_consist(sdp))
-			printk("GFS: fsid=%s: block = %"PRIu64"\n",
+			printk("GFS: fsid=%s: block = %llu\n",
 			       sdp->sd_fsname, block);
 		return;
 	}
