@@ -24,6 +24,7 @@
 #include <sys/wait.h>
 #include <arpa/inet.h>
 #include <dirent.h>
+#include <limits.h>
 #include <ctype.h>
 #define _GNU_SOURCE
 #include <getopt.h>
@@ -793,7 +794,7 @@ int main(int argc, char **argv){
       continue;
     case 'V':
       printf("%s %s (built %s %s)\n", argv[0],
-             GNBD_RELEASE_NAME, __DATE__, __TIME__);
+             RELEASE_VERSION, __DATE__, __TIME__);
       printf("%s\n", REDHAT_COPYRIGHT);
       return 0;
     default:
