@@ -884,7 +884,8 @@ static void decode_arguments(int argc, char **argv)
 			break;
 
 		case 'V':
-			printf("groupd (built %s %s)\n", __DATE__, __TIME__);
+			printf("groupd %s (built %s %s)\n",
+				RELEASE_VERSION, __DATE__, __TIME__);
 			/* printf("%s\n", REDHAT_COPYRIGHT); */
 			exit(EXIT_SUCCESS);
 			break;
