@@ -667,7 +667,7 @@ static int loop(void)
 				workfn = client[i].workfn;
 				workfn(i);
 			}
-			if (pollfd[i].revents & POLLHUP) {
+			if (pollfd[i].revents & (POLLHUP | POLLERR | POLLNVAL)) {
 				deadfn = client[i].deadfn;
 				deadfn(i);
 			}
@@ -784,7 +784,8 @@ static void decode_arguments(int argc, char **argv)
 			break;
 
 		case 'V':
-			printf("dlm_controld (built %s %s)\n", __DATE__, __TIME__);
+			printf("dlm_controld %s (built %s %s)\n",
+				RELEASE_VERSION, __DATE__, __TIME__);
 			/* printf("%s\n", REDHAT_COPYRIGHT); */
 			exit(EXIT_SUCCESS);
 			break;
