@@ -771,12 +771,8 @@ static int add_waiter(struct mountgroup *mg, struct resource *r,
 static void write_result(struct mountgroup *mg, struct gdlm_plock_info *in,
 			 int rv)
 {
-	int err;
-
 	in->rv = rv;
-	err = write(control_fd, in, sizeof(struct gdlm_plock_info));
-	if (err != sizeof(struct gdlm_plock_info))
-		log_error("plock result write err %d errno %d", err, errno);
+	write(control_fd, in, sizeof(struct gdlm_plock_info));
 }
 
 static void do_waiters(struct mountgroup *mg, struct resource *r)
